using System;
using System.Web.UI;
using System.Web.UI.WebControls;
using Microsoft.SharePoint;
using Microsoft.SharePoint.WebPartPages;
using Microsoft.SharePoint.Utilities;

namespace RH
{
    /// <summary>
    /// Description of the toolpart. Override the GetToolParts method
    /// in your Web Part class to invoke this toolpart. To establish
    /// a reference to the Web Part the user has selected, use the
    /// ParentToolPane.SelectedWebPart property.
    /// </summary>
    public class CustomToolPart : ToolPart
    {
        // declaring varialbles
        private string ListName;
        private string ViewName;
        private string TitleCSS;
        private string ControlCSS;

        // an event handler for the Init event
        private void CustomToolPart_Init(object sender, EventArgs e)
        {
            ListName = UniqueID + "message";
            ViewName = UniqueID + "View";
            TitleCSS = UniqueID + "Title";
            ControlCSS = UniqueID + "Control";
        }

        /// <summary>
        /// Constructor for the class. A great place to set Set
        /// default values for additional base class properties
        /// here.
        /// <summary>
        public CustomToolPart()
        {
            // Set default properties
            Title = "Display List Item";
            Init += new EventHandler(CustomToolPart_Init);
        }

        ///	<summary>
        ///	Called by the tool pane to apply property changes to
        /// the selected Web Part.
        ///	</summary>
        public override void ApplyChanges()
        {
            DisplayListItem wp1 = (DisplayListItem)ParentToolPane.SelectedWebPart;

            // Send the values to the Web Part.
            wp1.ListName = Page.Request.Form[ListName];
            wp1.ViewName = Page.Request.Form[ViewName];
            wp1.TitleCSS = SPEncode.HtmlEncode(Page.Request.Form[TitleCSS]);
            wp1.ControlCSS = SPEncode.HtmlEncode(Page.Request.Form[ControlCSS]);
        }

        /// <summary>
        ///	If the ApplyChanges method succeeds, this method is
        /// called by the tool pane to refresh the specified
        /// property values in the toolpart user interface.
        /// </summary>
        public override void SyncChanges()
        {
            // sync with the new property changes here
        }

        /// <summary>
        ///	Called by the tool pane if the user discards changes
        /// to the selected Web Part.
        /// </summary>
        public override void CancelChanges()
        {
        }

        /// <summary>
        /// Render this Tool part to the output parameter
        /// specified.
        /// </summary>
        /// <param name="output">
        /// The HTML writer to write out to
        /// </param>
        protected override void RenderToolPart(HtmlTextWriter output)
        {
            // Establish a reference to the Web Part.
            DisplayListItem wp1 = (DisplayListItem)ParentToolPane.SelectedWebPart;

            #region DropDownList List
            DropDownList ddList = new DropDownList();
            ddList.ID = ListName;
            foreach (SPList list in SPContext.Current.Web.Lists)
            {
                ListItem item = new ListItem();
                item.Text = list.Title;
                item.Value = list.Title;
                ddList.Items.Add(item);
            }
            ddList.SelectedValue = wp1.ListName;

            output.Write("Select a list: ");
            ddList.RenderControl(output);
            #endregion

            #region DropDownList View
            DropDownList ddView = new DropDownList();
            ddView.ID = ViewName;
            foreach (SPView view in SPContext.Current.Web.Lists[ddList.SelectedValue].Views)
            {
                ListItem item = new ListItem();
                item.Text = view.Title;
                item.Value = view.Title;
                ddView.Items.Add(item);
            }
            ddView.SelectedValue = wp1.ViewName;
            output.Write("<br/><br/>Press \"Apply\" after you changed your list to get a list of views for the list.<br/>Select a view:");
            ddView.RenderControl(output);
            #endregion

            output.Write("<br/><br/>CSS for title cell:<br/>");
            output.Write(String.Format("<input name='{0}' type='text' value='{1}'><br/>", TitleCSS, SPEncode.HtmlEncode(wp1.TitleCSS)));
            
            output.Write("<br/>CSS for control cell:<br/>");
            output.Write(String.Format("<input name='{0}' type='text' value='{1}'><br/>", ControlCSS, SPEncode.HtmlEncode(wp1.ControlCSS)));
        }
    }
}