using System;
using System.ComponentModel;
using System.Reflection;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Serialization;
using Microsoft.SharePoint;
using Microsoft.SharePoint.WebControls;
using Microsoft.SharePoint.WebPartPages;
using RH.SharePoint;
using WebPart=Microsoft.SharePoint.WebPartPages.WebPart;
using Microsoft.SharePoint.Utilities;

namespace RH
{
    /// <summary>
    /// <SafeControl  Assembly="RH.DisplayListItem, Version=1.0.0.0, Culture=neutral, PublicKeyToken=7850e867f7381714"  Namespace="RH"  TypeName="*"  Safe="True" />
    /// </summary>
    [ToolboxData("<{0}:DisplayListItem runat=server></{0}:DisplayListItem>"), XmlRoot(Namespace = "RH")]
    public class DisplayListItem : WebPart, IDesignTimeHtmlProvider 
    {
        private Table _Table;
        private Table _PagingTable;
        private ImageButton _ButtonBack;
        private ImageButton _ButtonForward;
        private string _ErrorMessage = String.Empty;
        private string _TitleCSS;
        private string _ControlCSS;
        private string _ListName;
        private string _ViewName;
        
        #region Properties
        [Browsable(false), Category("Miscellaneous"),
        DefaultValue(""),
        WebPartStorage(Storage.Personal),
        FriendlyName("Text"), Description("List Name")]
        public string ListName
        {
            get
            {
                return _ListName;
            }

            set
            {
                _ListName = value;
            }
        }

        [Browsable(false), Category("Miscellaneous"),
        DefaultValue(""),
        WebPartStorage(Storage.Personal),
        FriendlyName("Text"), Description("View Name")]
        public string ViewName
        {
            get
            {
                return GetViewName(_ListName, _ViewName);
            }

            set
            {
                _ViewName = value;
            }
        }

        [Browsable(false), Category("Miscellaneous"),
        DefaultValue(""),
        WebPartStorage(Storage.Personal),
        FriendlyName("Text"), Description("CSS for title")]
        public string TitleCSS
        {
            get
            {
                if (String.IsNullOrEmpty(_TitleCSS))
                    return "ms-formlabel";
                else
                    return _TitleCSS;
            }

            set
            {
                _TitleCSS = value;
            }
        }

        [Browsable(false), Category("Miscellaneous"),
        DefaultValue(""),
        WebPartStorage(Storage.Personal),
        FriendlyName("Text"), Description("CSS for controls")]
        public string ControlCSS
        {
            get
            {
                if (string.IsNullOrEmpty(_ControlCSS))
                    return "ms-formbody";
                else 
                    return _ControlCSS;
            }

            set
            {
                _ControlCSS = value;
            }
        }

        private int _PageNumber
        {
            get
            {
                if (ViewState["Page"] != null)
                {
                    return int.Parse(ViewState["Page"].ToString());
                } 
                else
                {
                    return 0;
                }
            }
            set
            {
                ViewState["Page"] = value;
            }
        }
        #endregion

        #region Filter
        private IWebPartField _provider;
        private object _fieldValue;

        private void GetFieldValue(object fieldValue)
        {
            _fieldValue = fieldValue;
        }

        public bool ConnectionPointEnabled
        {
            get
            {
                object o = ViewState["ConnectionPointEnabled"];
                return (o != null) ? (bool)o : true;
            }
            set
            {
                ViewState["ConnectionPointEnabled"] = value;
            }
        }

        [ConnectionConsumer("ListItemID", "Connpoint1", typeof(FieldConsumerConnectionPoint), AllowsMultipleConnections = true)]
        public void SetConnectionInterface(IWebPartField provider)
        {
            _provider = provider;
        }

        public class FieldConsumerConnectionPoint : ConsumerConnectionPoint
        {
            public FieldConsumerConnectionPoint(MethodInfo callbackMethod, Type interfaceType, Type controlType, string name, string id, bool allowsMultipleConnections)
                : base(callbackMethod, interfaceType, controlType, name, id, allowsMultipleConnections)
            {
            }

            public override bool GetEnabled(Control control)
            {
                return ((DisplayListItem)control).ConnectionPointEnabled;
            }
        }
        #endregion

        public override ToolPart[] GetToolParts()
        {
            ToolPart[] toolparts = new ToolPart[3];
            WebPartToolPart wptp = new WebPartToolPart();
            CustomPropertyToolPart custom = new CustomPropertyToolPart();
            toolparts[1] = custom;
            toolparts[2] = wptp;

            // This is the custom ToolPart.
            toolparts[0] = new CustomToolPart();

            return toolparts;
        }

        protected override void CreateChildControls()
        {
            _Table = new Table();
            _Table.ID = "myTable";
            Controls.Add(_Table);

            _PagingTable = new Table();
            _PagingTable.ID = "myPagingTable";
            Controls.Add(_PagingTable);

            _ButtonBack = new ImageButton();
            _ButtonBack.ID = "ButtonBack";
            _ButtonBack.Click += new ImageClickEventHandler(_ButtonBack_Click);
            Controls.Add(_ButtonBack);

            _ButtonForward = new ImageButton();
            _ButtonForward.ID = "ButtonForward";
            _ButtonForward.Click += new ImageClickEventHandler(_ButtonForward_Click);
            Controls.Add(_ButtonForward);

            base.CreateChildControls();
        }

        protected override void OnPreRender(EventArgs e)
        {
            base.OnPreRender(e);

            if (!String.IsNullOrEmpty(_ListName))
            {
                try
                {
                    SPWeb web = SPContext.Current.Web;
                    SPList list = web.Lists[_ListName];

                    //check for items in the list, and if the current user can read them
                    if (list.DoesUserHavePermissions(SPBasePermissions.ViewListItems) && list.ItemCount != 0)
                    {
                        SPListItem item;
                        if (_provider != null)
                        {
                            _provider.GetFieldValue(new FieldCallback(GetFieldValue));
                            int listItemID;
                            int.TryParse(SPEncode.HtmlEncode(_fieldValue.ToString()), out listItemID);

                            if (listItemID == 0)
                            {
                                _ErrorMessage += String.Format("\"{0}\" is not a valid ListItem Id", _fieldValue);
                                return;
                            }
                            item = GetListItem(list, listItemID);
                        }
                        else
                        {
                            // if we switch the list in the properties and the previous selected page is bigger than the new list.ItemCount, we have to reset the counter
                            if (_PageNumber > list.ItemCount - 1) _PageNumber = 0;
                            item = list.Items[_PageNumber];

                            if (list.ItemCount > 1)
                                CreatePagingTable(item);
                        }

                        if (item != null)
                            CreateTable(item);
                    }
                }
                catch (Exception ex)
                {
                    _ErrorMessage += String.Format("Error in OnPreRender:\r\n{0}\r\n", ex);
                }
            }
            else
            {
                _ErrorMessage += "Select a list in the webpart properties.";
            }
        }

        protected override void Render(HtmlTextWriter writer)
        {
            try
            {
                if (String.IsNullOrEmpty(_ErrorMessage))
                {
                    _Table.RenderControl(writer);
                }
                else
                {
                    HtmlGenericControl errorControl = new HtmlGenericControl("DIV");
                    errorControl.Attributes.Add("style", "padding: 20px; margin: 20px; border-style: groove; background-color: #FFFFFF;");
                    errorControl.InnerHtml = "<IMG src=\"/_layouts/images/error16by16.gif\" border=\"0\"/>" + _ErrorMessage;
                    errorControl.RenderControl(writer);
                }
            }
            catch (Exception ex)
            {
                HtmlGenericControl errorControl = new HtmlGenericControl("DIV");
                errorControl.Attributes.Add("style", "padding: 20px; margin: 20px; border-style: groove; background-color: #FFFFFF;");
                errorControl.InnerHtml = "<IMG src=\"/_layouts/images/error16by16.gif\" border=\"0\"/>" + ex;
                errorControl.RenderControl(writer);
            }
        }

        #region Create Tables
        private void CreateTable(SPListItem item)
        {
            try
            {
                SPView listView = item.ParentList.Views[ViewName];
                for (int i = 0; i < listView.ViewFields.Count; i++)
                {
                    string fieldName = listView.ViewFields[i];
                    SPField field = item.ParentList.Fields.GetField(fieldName);

                    TableRow row = new TableRow();
                    TableCell cell1 = new TableCell();
                    cell1.CssClass = _TitleCSS;
                    cell1.Text = field.Title;
                    row.Cells.Add(cell1);

                    TableCell cell2 = new TableCell();
                    cell2.CssClass = _ControlCSS;
                    Control cntrl = SharePointWebControls.GetSharePointControls(field, item.ParentList, item, SPControlMode.Display);
                    if (cntrl != null)
                    {
                        cell2.Controls.Add(cntrl);
                        row.Cells.Add(cell2);

                        _Table.Rows.Add(row);
                    }
                }

                TableRow pagingRow = new TableRow();
                TableCell cell = new TableCell();
                cell.ColumnSpan = 2;
                cell.Controls.Add(_PagingTable);
                pagingRow.Cells.Add(cell);
                _Table.Rows.Add(pagingRow);
            }
            catch (Exception ex)
            {
                _ErrorMessage += ex + "\r\n";
            }
        }

        private void CreatePagingTable(SPListItem item)
        {
            int itemCount = item.ParentList.ItemCount;
            if (_PageNumber > itemCount - 1)
            {
                _PageNumber = itemCount;
            }

            // finally create the paging table
            _PagingTable.CssClass = "ms-bottompaging";
            _PagingTable.Width = Unit.Percentage(100);
            TableRow row = new TableRow();
            TableCell cell = new TableCell();
            cell.CssClass = "ms-bottompagingline1";
            cell.ColumnSpan = 3;
            row.Cells.Add(cell);
            _PagingTable.Rows.Add(row);

            row = new TableRow();
            cell = new TableCell();
            cell.CssClass = "ms-bottompagingline2";
            cell.ColumnSpan = 3;
            row.Cells.Add(cell);
            _PagingTable.Rows.Add(row);

            row = new TableRow();
            cell = new TableCell();
            cell.Width = Unit.Percentage(49);
            if (_PageNumber != 0)
            {
                cell.HorizontalAlign = HorizontalAlign.Right;
                _ButtonBack.ImageUrl = "/_layouts/images/prev.gif";
                cell.Controls.Add(_ButtonBack);
            }
            else cell.Text = "&nbsp;";
            row.Cells.Add(cell);

            cell = new TableCell();
            cell.CssClass = "ms-paging";
            cell.Text = String.Format("<nobr>{0} - {1}</nobr>", _PageNumber + 1, item.ParentList.Items.Count);
            row.Cells.Add(cell);

            cell = new TableCell();
            cell.Width = Unit.Percentage(49);
            if (_PageNumber != itemCount - 1)
            {
                cell.HorizontalAlign = HorizontalAlign.Left;
                _ButtonForward.ImageUrl = "/_layouts/images/next.gif";
                cell.Controls.Add(_ButtonForward);
            }
            else cell.Text = "&nbsp;";
            row.Cells.Add(cell);

            _PagingTable.Rows.Add(row);
        }
        #endregion

        /// <summary>
        /// Returns the viewname if the view exists in the selected list. Otherwise returns the defaultview or string.empty
        /// </summary>
        /// <param name="_ListName"></param>
        /// <param name="_ViewName"></param>
        /// <returns></returns>
        private static string GetViewName(string _ListName, string _ViewName)
        {
            SPList list = SPContext.Current.Web.Lists[_ListName];

            if (!String.IsNullOrEmpty(_ListName) && !String.IsNullOrEmpty(_ViewName))
            {
                foreach (SPView view in list.Views)
                {
                    if (view.Title == _ViewName)
                    {
                        return _ViewName;
                    }
                }
                return list.DefaultView.Title;
            }
            else if(!String.IsNullOrEmpty(_ListName))
            {
                return list.DefaultView.Title;
            }

            // if the list is null, we can not return a defaultview
            return string.Empty;
        }

        /// <summary>
        /// If there is a listitem with this id, it will be returned. Otherwise you'll get a null
        /// </summary>
        /// <param name="List"></param>
        /// <param name="ListItemID"></param>
        /// <returns></returns>
        private SPListItem GetListItem(SPList List, int ListItemID)
        {
            try
            {
                string defaultView = List.DefaultView.Title;
                SPQuery query = new SPQuery(List.Views[defaultView]);

                string caml = "<Where><Eq><FieldRef Name='ID'/><Value Type='Counter'>" + ListItemID + "</Value></Eq></Where>";
                query.Query = caml;
                SPListItemCollection results = List.GetItems(query);

                if (results.Count == 1)
                {
                    return results[0];
                } 
            }
            catch (Exception ex)
            {
                _ErrorMessage += String.Format("List \"{0}\" does not contain an item with the id \"{1}\".<br/>{2}", List.Title, ListItemID, ex.Message);
            }

            return null;
        }

        public string GetDesignTimeHtml()
        {
            return SPEncode.HtmlEncode("This Webpart will display a single ListItem. It can receive an ItemID to display a particullar Item.");
        }

        #region Handler
        void _ButtonForward_Click(object sender, ImageClickEventArgs e)
        {
            _PageNumber++;
        }

        void _ButtonBack_Click(object sender, ImageClickEventArgs e)
        {
            _PageNumber--;

            if (_PageNumber < 0)
            {
                _PageNumber = 0;
            }
        }
        #endregion
    }
}
